@@ -26,6 +26,8 @@
 
 extern BOOL in_client;
 extern pstring user_socket_options;
+extern char *optarg;
+extern int optind;
 
 static pstring credentials;
 static pstring my_netbios_name;
@@ -149,7 +151,7 @@ static struct cli_state *do_connection(char *the_service)
 	if (have_ip) ip = dest_ip;
 
 	/* have to open a new connection */
-	if (!(c=cli_initialise(NULL)) || (cli_set_port(c, smb_port) != smb_port) ||
+	if (!(c=cli_initialise()) || (cli_set_port(c, smb_port) != smb_port) ||
 	    !cli_connect(c, server_n, &ip)) {
 		DEBUG(0,("%d: Connection to %s failed\n", sys_getpid(), server_n));
 		if (c) {
@@ -211,14 +213,14 @@ static struct cli_state *do_connection(char *the_service)
 		c->force_dos_errors = True;
 	}
 
-	if (!cli_session_setup(c, username, 
-			       password, strlen(password),
-			       password, strlen(password),
-			       workgroup)) {
+	if (!NT_STATUS_IS_OK(cli_session_setup(c, username, 
+					       password, strlen(password),
+					       password, strlen(password),
+					       workgroup))) {
 		/* if a password was not supplied then try again with a
 			null username */
 		if (password[0] || !username[0] ||
-				!cli_session_setup(c, "", "", 0, "", 0, workgroup)) {
+		    !NT_STATUS_IS_OK(cli_session_setup(c, "", "", 0, "", 0, workgroup))) {
 			DEBUG(0,("%d: session setup failed: %s\n",
 				sys_getpid(), cli_errstr(c)));
 			cli_shutdown(c);
@@ -386,6 +388,9 @@ static void send_fs_socket(char *the_service, char *mount_point, struct cli_stat
 		c = NULL;
 
 		if (!closed) {
+			/* close the name cache so that close_our_files() doesn't steal its FD */
+			namecache_shutdown();
+
 			/* redirect stdout & stderr since we can't know that
 			   the library functions we use are using DEBUG. */
 			if ( (fd = open("/dev/null", O_WRONLY)) < 0)
@@ -664,6 +669,9 @@ static void usage(void)
 
 	printf("Version %s\n\n",SAMBA_VERSION_STRING);
 
+	printf("Please be aware that smbfs is deprecated in favor of "
+	       "cifs\n\n");
+
 	printf(
 "Options:\n\
       username=<arg>                  SMB username\n\
@@ -709,7 +717,6 @@ static void parse_mount_smb(int argc, char **argv)
 	int opt;
 	char *opts;
 	char *opteq;
-	extern char *optarg;
 	int val;
 	char *p;
 
@@ -859,12 +866,12 @@ static void parse_mount_smb(int argc, char **argv)
 ****************************************************************************/
  int main(int argc,char *argv[])
 {
-	extern char *optarg;
-	extern int optind;
 	char *p;
 
 	DEBUGLEVEL = 1;
 
+	load_case_tables();
+
 	/* here we are interactive, even if run from autofs */
 	setup_logging("mount.smbfs",True);
 
@@ -907,7 +914,7 @@ static void parse_mount_smb(int argc, char **argv)
 		pstrcpy(username,getenv("LOGNAME"));
 	}
 
-	if (!lp_load(dyn_CONFIGFILE,True,False,False)) {
+	if (!lp_load(dyn_CONFIGFILE,True,False,False,True)) {
 		fprintf(stderr, "Can't load %s - run testparm to debug it\n", 
 			dyn_CONFIGFILE);
 	}
