@@ -180,6 +180,9 @@ static const smb_socket_option socket_options[] = {
 #endif
 #ifdef SO_RCVTIMEO
   {"SO_RCVTIMEO",       SOL_SOCKET,    SO_RCVTIMEO,     0,                 OPT_INT},
+#endif
+#ifdef TCP_FASTACK
+  {"TCP_FASTACK",       IPPROTO_TCP,   TCP_FASTACK,     0,                 OPT_INT},
 #endif
   {NULL,0,0,0,0}};
 
@@ -275,7 +278,12 @@ ssize_t read_udp_socket(int fd,char *buf,size_t len)
 	memset((char *)&lastip,'\0',sizeof(lastip));
 	ret = (ssize_t)sys_recvfrom(fd,buf,len,0,(struct sockaddr *)&sock,&socklen);
 	if (ret <= 0) {
-		DEBUG(2,("read socket failed. ERRNO=%s\n",strerror(errno)));
+		/* Don't print a low debug error for a non-blocking socket. */
+		if (errno == EAGAIN) {
+			DEBUG(10,("read socket returned EAGAIN. ERRNO=%s\n",strerror(errno)));
+		} else {
+			DEBUG(2,("read socket failed. ERRNO=%s\n",strerror(errno)));
+		}
 		return(0);
 	}
 
@@ -659,8 +667,6 @@ BOOL receive_smb_raw(int fd, char *buffer, unsigned int timeout)
 
 	smb_read_error = 0;
 
-	memset(buffer,'\0',smb_size + 100);
-
 	len = read_smb_length_return_keepalive(fd,buffer,timeout);
 	if (len < 0) {
 		DEBUG(10,("receive_smb_raw: length < 0!\n"));
@@ -705,8 +711,9 @@ BOOL receive_smb_raw(int fd, char *buffer, unsigned int timeout)
 		}
 
 		if (ret != len) {
-			if (smb_read_error == 0)
+			if (smb_read_error == 0) {
 				smb_read_error = READ_ERROR;
+			}
 			return False;
 		}
 		
@@ -967,7 +974,10 @@ BOOL open_any_socket_out(struct sockaddr_in *addrs, int num_addrs,
 		}
 
 		if (errno == EINPROGRESS || errno == EALREADY ||
-		    errno == EAGAIN) {
+#ifdef EISCONN
+			errno == EISCONN ||
+#endif
+		    errno == EAGAIN || errno == EINTR) {
 			/* These are the error messages that something is
 			   progressing. */
 			good_connect = True;
@@ -1001,7 +1011,7 @@ BOOL open_any_socket_out(struct sockaddr_in *addrs, int num_addrs,
 	tv.tv_sec = 0;
 	tv.tv_usec = connect_loop;
 
-	res = sys_select(maxfd+1, &r_fds, &wr_fds, NULL, &tv);
+	res = sys_select_intr(maxfd+1, &r_fds, &wr_fds, NULL, &tv);
 
 	if (res < 0)
 		goto done;
