@@ -23,6 +23,7 @@
 extern fstring remote_proto;
 extern enum protocol_types Protocol;
 extern int max_recv;
+
 BOOL global_encrypted_passwords_negotiated = False;
 BOOL global_spnego_negotiated = False;
 struct auth_context *negprot_global_auth_context = NULL;
@@ -32,7 +33,8 @@ static void get_challenge(char buff[8])
 	NTSTATUS nt_status;
 	const uint8 *cryptkey;
 
-	/* We might be called more than once, muliple negprots are premitted */
+	/* We might be called more than once, multiple negprots are
+	 * permitted */
 	if (negprot_global_auth_context) {
 		DEBUG(3, ("get challenge: is this a secondary negprot?  negprot_global_auth_context is non-NULL!\n"));
 		(negprot_global_auth_context->free)(&negprot_global_auth_context);
@@ -115,9 +117,9 @@ static int reply_lanman1(char *inbuf, char *outbuf)
 	SSVAL(outbuf,smb_vwv5,raw); /* tell redirector we support
 		readbraw writebraw (possibly) */
 	SIVAL(outbuf,smb_vwv6,sys_getpid());
-	SSVAL(outbuf,smb_vwv10, TimeDiff(t)/60);
+	SSVAL(outbuf,smb_vwv10, set_server_zone_offset(t)/60);
 
-	put_dos_date(outbuf,smb_vwv8,t);
+	srv_put_dos_date(outbuf,smb_vwv8,t);
 
 	return (smb_len(outbuf)+4);
 }
@@ -157,8 +159,8 @@ static int reply_lanman2(char *inbuf, char *outbuf)
 	SSVAL(outbuf,smb_vwv3,lp_maxmux()); 
 	SSVAL(outbuf,smb_vwv4,1);
 	SSVAL(outbuf,smb_vwv5,raw); /* readbraw and/or writebraw */
-	SSVAL(outbuf,smb_vwv10, TimeDiff(t)/60);
-	put_dos_date(outbuf,smb_vwv8,t);
+	SSVAL(outbuf,smb_vwv10, set_server_zone_offset(t)/60);
+	srv_put_dos_date(outbuf,smb_vwv8,t);
 
 	return (smb_len(outbuf)+4);
 }
@@ -167,51 +169,59 @@ static int reply_lanman2(char *inbuf, char *outbuf)
  Generate the spnego negprot reply blob. Return the number of bytes used.
 ****************************************************************************/
 
-static int negprot_spnego(char *p)
+static DATA_BLOB negprot_spnego(void)
 {
 	DATA_BLOB blob;
 	nstring dos_name;
 	fstring unix_name;
-	uint8 guid[17];
+#ifdef DEVELOPER
+	size_t slen;
+#endif
+	char guid[17];
 	const char *OIDs_krb5[] = {OID_KERBEROS5,
 				   OID_KERBEROS5_OLD,
 				   OID_NTLMSSP,
 				   NULL};
 	const char *OIDs_plain[] = {OID_NTLMSSP, NULL};
-	int len;
 
 	global_spnego_negotiated = True;
 
-	ZERO_STRUCT(guid);
+	memset(guid, '\0', sizeof(guid));
 
 	safe_strcpy(unix_name, global_myname(), sizeof(unix_name)-1);
 	strlower_m(unix_name);
 	push_ascii_nstring(dos_name, unix_name);
-	safe_strcpy((char *)guid, dos_name, sizeof(guid)-1);
+	safe_strcpy(guid, dos_name, sizeof(guid)-1);
 
 #ifdef DEVELOPER
-	/* valgrind fixer... */
-	{
-		size_t sl = strlen(guid);
-		if (sizeof(guid)-sl)
-			memset(&guid[sl], '\0', sizeof(guid)-sl);
+	/* Fix valgrind 'uninitialized bytes' issue. */
+	slen = strlen(dos_name);
+	if (slen < sizeof(guid)) {
+		memset(guid+slen, '\0', sizeof(guid) - slen);
 	}
 #endif
 
-#if 0
 	/* strangely enough, NT does not sent the single OID NTLMSSP when
 	   not a ADS member, it sends no OIDs at all
 
-	   we can't do this until we teach our sesssion setup parser to know
-	   about raw NTLMSSP (clients send no ASN.1 wrapping if we do this)
+	   OLD COMMENT : "we can't do this until we teach our sesssion setup parser to know
+		   about raw NTLMSSP (clients send no ASN.1 wrapping if we do this)"
+
+	   Our sessionsetup code now handles raw NTLMSSP connects, so we can go
+	   back to doing what W2K3 does here. This is needed to make PocketPC 2003
+	   CIFS connections work with SPNEGO. See bugzilla bugs #1828 and #3133
+	   for details. JRA.
+
 	*/
-	if (lp_security() != SEC_ADS) {
-		memcpy(p, guid, 16);
-		return 16;
-	}
-#endif
+
 	if (lp_security() != SEC_ADS && !lp_use_kerberos_keytab()) {
+#if 0
+		/* Code for PocketPC client */
+		blob = data_blob(guid, 16);
+#else
+		/* Code for standalone WXP client */
 		blob = spnego_gen_negTokenInit(guid, OIDs_plain, "NONE");
+#endif
 	} else {
 		fstring myname;
 		char *host_princ_s = NULL;
@@ -221,10 +231,8 @@ static int negprot_spnego(char *p)
 		blob = spnego_gen_negTokenInit(guid, OIDs_krb5, host_princ_s);
 		SAFE_FREE(host_princ_s);
 	}
-	memcpy(p, blob.data, blob.length);
-	len = blob.length;
-	data_blob_free(&blob);
-	return len;
+
+	return blob;
 }
 
 /****************************************************************************
@@ -238,12 +246,24 @@ static int reply_nt1(char *inbuf, char *outbuf)
 		CAP_LEVEL_II_OPLOCKS;
 
 	int secword=0;
-	time_t t = time(NULL);
 	char *p, *q;
 	BOOL negotiate_spnego = False;
+	time_t t = time(NULL);
 
 	global_encrypted_passwords_negotiated = lp_encrypted_passwords();
 
+	/* Check the flags field to see if this is Vista.
+	   WinXP sets it and Vista does not. But we have to 
+	   distinguish from NT which doesn't set it either. */
+
+	if ( (SVAL(inbuf, smb_flg2) & FLAGS2_EXTENDED_SECURITY) &&
+		((SVAL(inbuf, smb_flg2) & FLAGS2_UNKNOWN_BIT4) == 0) ) 
+	{
+		if (get_remote_arch() != RA_SAMBA) {
+			set_remote_arch( RA_VISTA );
+		}
+	}
+
 	/* do spnego in user level security if the client
 	   supports it and we can do encrypted passwords */
 	
@@ -296,7 +316,7 @@ static int reply_nt1(char *inbuf, char *outbuf)
 		} else {
 			DEBUG(0,("reply_nt1: smb signing is incompatible with share level security !\n"));
 			if (lp_server_signing() == Required) {
-				exit_server("reply_nt1: smb signing required and share level security selected.");
+				exit_server_cleanly("reply_nt1: smb signing required and share level security selected.");
 			}
 		}
 	}
@@ -314,7 +334,7 @@ static int reply_nt1(char *inbuf, char *outbuf)
 	SIVAL(outbuf,smb_vwv7+1,sys_getpid()); /* session key */
 	SIVAL(outbuf,smb_vwv9+1,capabilities); /* capabilities */
 	put_long_date(outbuf+smb_vwv11+1,t);
-	SSVALS(outbuf,smb_vwv15+1,TimeDiff(t)/60);
+	SSVALS(outbuf,smb_vwv15+1,set_server_zone_offset(t)/60);
 	
 	p = q = smb_buf(outbuf);
 	if (!negotiate_spnego) {
@@ -323,17 +343,24 @@ static int reply_nt1(char *inbuf, char *outbuf)
 			/* note that we do not send a challenge at all if
 			   we are using plaintext */
 			get_challenge(p);
-			SSVALS(outbuf,smb_vwv16+1,8);
+			SCVAL(outbuf,smb_vwv16+1,8);
 			p += 8;
 		}
 		p += srvstr_push(outbuf, p, lp_workgroup(), -1, 
 				 STR_UNICODE|STR_TERMINATE|STR_NOALIGN);
 		DEBUG(3,("not using SPNEGO\n"));
 	} else {
-		int len = negprot_spnego(p);
-		
-		SSVALS(outbuf,smb_vwv16+1,len);
-		p += len;
+		DATA_BLOB spnego_blob = negprot_spnego();
+
+		if (spnego_blob.data == NULL) {
+			return ERROR_NT(NT_STATUS_NO_MEMORY);
+		}
+
+		memcpy(p, spnego_blob.data, spnego_blob.length);
+		p += spnego_blob.length;
+		data_blob_free(&spnego_blob);
+
+		SCVAL(outbuf,smb_vwv16+1, 0);
 		DEBUG(3,("using SPNEGO\n"));
 	}
 	
@@ -373,6 +400,15 @@ protocol [LM1.2X002]
 protocol [LANMAN2.1]
 protocol [NT LM 0.12]
 
+Vista:
+protocol [PC NETWORK PROGRAM 1.0]
+protocol [LANMAN1.0]
+protocol [Windows for Workgroups 3.1a]
+protocol [LM1.2X002]
+protocol [LANMAN2.1]
+protocol [NT LM 0.12]
+protocol [SMB 2.001]
+
 OS/2:
 protocol [PC NETWORK PROGRAM 1.0]
 protocol [XENIX CORE]
@@ -386,18 +422,19 @@ protocol [LANMAN2.1]
   *
   * This appears to be the matrix of which protocol is used by which
   * MS product.
-       Protocol                       WfWg    Win95   WinNT  Win2K  OS/2
-       PC NETWORK PROGRAM 1.0          1       1       1      1      1
+       Protocol                       WfWg    Win95   WinNT  Win2K  OS/2 Vista
+       PC NETWORK PROGRAM 1.0          1       1       1      1      1     1
        XENIX CORE                                      2             2
        MICROSOFT NETWORKS 3.0          2       2       
        DOS LM1.2X002                   3       3       
        MICROSOFT NETWORKS 1.03                         3
        DOS LANMAN2.1                   4       4       
-       LANMAN1.0                                       4      2      3
-       Windows for Workgroups 3.1a     5       5       5      3
-       LM1.2X002                                       6      4      4
-       LANMAN2.1                                       7      5      5
-       NT LM 0.12                              6       8      6
+       LANMAN1.0                                       4      2      3     2
+       Windows for Workgroups 3.1a     5       5       5      3            3
+       LM1.2X002                                       6      4      4     4
+       LANMAN2.1                                       7      5      5     5
+       NT LM 0.12                              6       8      6            6
+       SMB 2.001                                                           7
   *
   *  tim@fsg.com 09/29/95
   *  Win2K added by matty 17/7/99
@@ -410,6 +447,7 @@ protocol [LANMAN2.1]
 #define ARCH_OS2      0x14     /* Again OS/2 is like NT */
 #define ARCH_SAMBA    0x20
 #define ARCH_CIFSFS   0x40
+#define ARCH_VISTA    0x8C     /* Vista is like XP/2K */
  
 #define ARCH_ALL      0x7F
  
@@ -436,6 +474,7 @@ static const struct {
 
 /****************************************************************************
  Reply to a negprot.
+ conn POINTER CAN BE NULL HERE !
 ****************************************************************************/
 
 int reply_negprot(connection_struct *conn, 
@@ -456,7 +495,7 @@ int reply_negprot(connection_struct *conn,
 
 	if (done_negprot) {
 		END_PROFILE(SMBnegprot);
-		exit_server("multiple negprot's are not permitted");
+		exit_server_cleanly("multiple negprot's are not permitted");
 	}
 	done_negprot = True;
 
@@ -472,6 +511,8 @@ int reply_negprot(connection_struct *conn,
 			arch &= ( ARCH_WFWG | ARCH_WIN95 );
 		else if (strcsequal(p,"NT LM 0.12"))
 			arch &= ( ARCH_WIN95 | ARCH_WINNT | ARCH_WIN2K | ARCH_CIFSFS);
+		else if (strcsequal(p,"SMB 2.001"))
+			arch = ARCH_VISTA;		
 		else if (strcsequal(p,"LANMAN2.1"))
 			arch &= ( ARCH_WINNT | ARCH_WIN2K | ARCH_OS2 );
 		else if (strcsequal(p,"LM1.2X002"))
@@ -516,7 +557,13 @@ int reply_negprot(connection_struct *conn,
 				set_remote_arch(RA_WINNT);
 			break;
 		case ARCH_WIN2K:
-			set_remote_arch(RA_WIN2K);
+			/* Vista may have been set in the negprot so don't 
+			   override it here */
+			if ( get_remote_arch() != RA_VISTA )
+				set_remote_arch(RA_WIN2K);
+			break;
+		case ARCH_VISTA:
+			set_remote_arch(RA_VISTA);
 			break;
 		case ARCH_OS2:
 			set_remote_arch(RA_OS2);
@@ -565,7 +612,8 @@ int reply_negprot(connection_struct *conn,
 	DEBUG( 5, ( "negprot index=%d\n", choice ) );
 
 	if ((lp_server_signing() == Required) && (Protocol < PROTOCOL_NT1)) {
-		exit_server("SMB signing is required and client negotiated a downlevel protocol");
+		exit_server_cleanly("SMB signing is required and "
+			"client negotiated a downlevel protocol");
 	}
 
 	END_PROFILE(SMBnegprot);
