@@ -33,8 +33,12 @@ static int die_on_error;
 static int NumLoops = 0;
 static int ignore_dot_errors = 0;
 
+extern char *optarg;
+extern int optind;
+extern BOOL AllowDebugChange;
+
 /* a test fn for LANMAN mask support */
-int ms_fnmatch_lanman_core(const char *pattern, const char *string)
+static int ms_fnmatch_lanman_core(const char *pattern, const char *string)
 {
 	const char *p = pattern, *n = string;
 	char c;
@@ -108,7 +112,7 @@ next:
 	return 0;
 }
 
-int ms_fnmatch_lanman(const char *pattern, const char *string)
+static int ms_fnmatch_lanman(const char *pattern, const char *string)
 {
 	if (!strpbrk(pattern, "?*<>\"")) {
 		if (strcmp(string,"..") == 0) 
@@ -159,7 +163,7 @@ static char *reg_test(struct cli_state *cli, char *pattern, char *long_name, cha
 /***************************************************** 
 return a connection to a server
 *******************************************************/
-struct cli_state *connect_one(char *share)
+static struct cli_state *connect_one(char *share)
 {
 	struct cli_state *c;
 	struct nmb_name called, calling;
@@ -184,7 +188,7 @@ struct cli_state *connect_one(char *share)
         zero_ip(&ip);
 
 	/* have to open a new connection */
-	if (!(c=cli_initialise(NULL)) || !cli_connect(c, server_n, &ip)) {
+	if (!(c=cli_initialise()) || !cli_connect(c, server_n, &ip)) {
 		DEBUG(0,("Connection to %s failed\n", server_n));
 		return NULL;
 	}
@@ -216,10 +220,10 @@ struct cli_state *connect_one(char *share)
 		}
 	}
 
-	if (!cli_session_setup(c, username, 
-			       password, strlen(password),
-			       password, strlen(password),
-			       lp_workgroup())) {
+	if (!NT_STATUS_IS_OK(cli_session_setup(c, username, 
+					       password, strlen(password),
+					       password, strlen(password),
+					       lp_workgroup()))) {
 		DEBUG(0,("session setup failed: %s\n", cli_errstr(c)));
 		return NULL;
 	}
@@ -426,9 +430,6 @@ static void usage(void)
 {
 	char *share;
 	struct cli_state *cli;	
-	extern char *optarg;
-	extern int optind;
-	extern BOOL AllowDebugChange;
 	int opt;
 	char *p;
 	int seed;
@@ -454,7 +455,7 @@ static void usage(void)
 	argc -= 1;
 	argv += 1;
 
-	lp_load(dyn_CONFIGFILE,True,False,False);
+	lp_load(dyn_CONFIGFILE,True,False,False,True);
 	load_interfaces();
 
 	if (getenv("USER")) {
