@@ -24,8 +24,11 @@
 /********************************************************************
 ********************************************************************/
 
-static WERROR query_service_state( struct cli_state *cli, TALLOC_CTX *mem_ctx, 
-                             POLICY_HND *hSCM, const char *service, uint32 *state )
+static WERROR query_service_state(struct rpc_pipe_client *pipe_hnd,
+				TALLOC_CTX *mem_ctx, 
+				POLICY_HND *hSCM,
+				const char *service,
+				uint32 *state )
 {
 	POLICY_HND hService;
 	SERVICE_STATUS service_status;
@@ -33,20 +36,20 @@ static WERROR query_service_state( struct cli_state *cli, TALLOC_CTX *mem_ctx,
 	
 	/* now cycle until the status is actually 'watch_state' */
 	
-	result = cli_svcctl_open_service( cli, mem_ctx, hSCM, &hService, 
+	result = rpccli_svcctl_open_service(pipe_hnd, mem_ctx, hSCM, &hService, 
 		service, SC_RIGHT_SVC_QUERY_STATUS );
 
 	if ( !W_ERROR_IS_OK(result) ) {
-		d_printf("Failed to open service.  [%s]\n", dos_errstr(result));
+		d_fprintf(stderr, "Failed to open service.  [%s]\n", dos_errstr(result));
 		return result;
 	}
 
-	result = cli_svcctl_query_status( cli, mem_ctx, &hService, &service_status  );
+	result = rpccli_svcctl_query_status(pipe_hnd, mem_ctx, &hService, &service_status  );
 	if ( W_ERROR_IS_OK(result) ) {
 		*state = service_status.state;
 	}
 	
-	cli_svcctl_close_service( cli, mem_ctx, &hService );
+	rpccli_svcctl_close_service(pipe_hnd, mem_ctx, &hService );
 	
 	return result;
 }
@@ -54,9 +57,12 @@ static WERROR query_service_state( struct cli_state *cli, TALLOC_CTX *mem_ctx,
 /********************************************************************
 ********************************************************************/
 
-static WERROR watch_service_state( struct cli_state *cli, TALLOC_CTX *mem_ctx, 
-                                   POLICY_HND *hSCM, const char *service, 
-				   uint32 watch_state, uint32 *final_state )
+static WERROR watch_service_state(struct rpc_pipe_client *pipe_hnd,
+				TALLOC_CTX *mem_ctx, 
+				POLICY_HND *hSCM,
+				const char *service, 
+				uint32 watch_state,
+				uint32 *final_state )
 {
 	uint32 i;
 	uint32 state = 0;
@@ -67,14 +73,14 @@ static WERROR watch_service_state( struct cli_state *cli, TALLOC_CTX *mem_ctx,
 	while ( (state != watch_state ) && i<30 ) {
 		/* get the status */
 
-		result = query_service_state( cli, mem_ctx, hSCM, service, &state  );
+		result = query_service_state(pipe_hnd, mem_ctx, hSCM, service, &state  );
 		if ( !W_ERROR_IS_OK(result) ) {
 			break;
 		}
 		
 		d_printf(".");
 		i++;
-		usleep( 100 );
+		sys_usleep( 100 );
 	}
 	d_printf("\n");
 	
@@ -86,9 +92,12 @@ static WERROR watch_service_state( struct cli_state *cli, TALLOC_CTX *mem_ctx,
 /********************************************************************
 ********************************************************************/
 
-static WERROR control_service( struct cli_state *cli, TALLOC_CTX *mem_ctx, 
-                             POLICY_HND *hSCM, const char *service, 
-			     uint32 control, uint32 watch_state )
+static WERROR control_service(struct rpc_pipe_client *pipe_hnd,
+				TALLOC_CTX *mem_ctx, 
+				POLICY_HND *hSCM,
+				const char *service, 
+				uint32 control,
+				uint32 watch_state )
 {
 	POLICY_HND hService;
 	WERROR result = WERR_GENERAL_FAILURE;
@@ -97,32 +106,32 @@ static WERROR control_service( struct cli_state *cli, TALLOC_CTX *mem_ctx,
 	
 	/* Open the Service */
 	
-	result = cli_svcctl_open_service( cli, mem_ctx, hSCM, &hService, 
+	result = rpccli_svcctl_open_service(pipe_hnd, mem_ctx, hSCM, &hService, 
 		service, (SC_RIGHT_SVC_STOP|SC_RIGHT_SVC_PAUSE_CONTINUE) );
 
 	if ( !W_ERROR_IS_OK(result) ) {
-		d_printf("Failed to open service.  [%s]\n", dos_errstr(result));
+		d_fprintf(stderr, "Failed to open service.  [%s]\n", dos_errstr(result));
 		goto done;
 	}
 	
 	/* get the status */
 
-	result = cli_svcctl_control_service( cli, mem_ctx, &hService, 
+	result = rpccli_svcctl_control_service(pipe_hnd, mem_ctx, &hService, 
 		control, &service_status  );
 		
 	if ( !W_ERROR_IS_OK(result) ) {
-		d_printf("Control service request failed.  [%s]\n", dos_errstr(result));
+		d_fprintf(stderr, "Control service request failed.  [%s]\n", dos_errstr(result));
 		goto done;
 	}
 	
 	/* loop -- checking the state until we are where we want to be */
 	
-	result = watch_service_state( cli, mem_ctx, hSCM, service, watch_state, &state );
+	result = watch_service_state(pipe_hnd, mem_ctx, hSCM, service, watch_state, &state );
 		
 	d_printf("%s service is %s.\n", service, svc_status_string(state));
 
 done:	
-	cli_svcctl_close_service( cli, mem_ctx, &hService  );
+	rpccli_svcctl_close_service(pipe_hnd, mem_ctx, &hService  );
 		
 	return result;
 }	
@@ -130,9 +139,13 @@ done:
 /********************************************************************
 ********************************************************************/
 
-static NTSTATUS rpc_service_list_internal( const DOM_SID *domain_sid, const char *domain_name, 
-                                           struct cli_state *cli, TALLOC_CTX *mem_ctx, 
-                                           int argc, const char **argv )
+static NTSTATUS rpc_service_list_internal(const DOM_SID *domain_sid,
+					const char *domain_name, 
+					struct cli_state *cli,
+					struct rpc_pipe_client *pipe_hnd,
+					TALLOC_CTX *mem_ctx, 
+					int argc,
+					const char **argv )
 {
 	POLICY_HND hSCM;
 	ENUM_SERVICES_STATUS *services;
@@ -147,17 +160,17 @@ static NTSTATUS rpc_service_list_internal( const DOM_SID *domain_sid, const char
 		return NT_STATUS_OK;
 	}
 
-	result = cli_svcctl_open_scm( cli, mem_ctx, &hSCM, SC_RIGHT_MGR_ENUMERATE_SERVICE  );
+	result = rpccli_svcctl_open_scm(pipe_hnd, mem_ctx, &hSCM, SC_RIGHT_MGR_ENUMERATE_SERVICE  );
 	if ( !W_ERROR_IS_OK(result) ) {
-		d_printf("Failed to open Service Control Manager.  [%s]\n", dos_errstr(result));
+		d_fprintf(stderr, "Failed to open Service Control Manager.  [%s]\n", dos_errstr(result));
 		return werror_to_ntstatus(result);
 	}
 	
-	result = cli_svcctl_enumerate_services( cli, mem_ctx, &hSCM, SVCCTL_TYPE_WIN32,
+	result = rpccli_svcctl_enumerate_services(pipe_hnd, mem_ctx, &hSCM, SVCCTL_TYPE_WIN32,
 		SVCCTL_STATE_ALL, &num_services, &services );
 	
 	if ( !W_ERROR_IS_OK(result) ) {
-		d_printf("Failed to enumerate services.  [%s]\n", dos_errstr(result));
+		d_fprintf(stderr, "Failed to enumerate services.  [%s]\n", dos_errstr(result));
 		goto done;
 	}
 	
@@ -172,7 +185,7 @@ static NTSTATUS rpc_service_list_internal( const DOM_SID *domain_sid, const char
 	}
 
 done:	
-	cli_svcctl_close_service( cli, mem_ctx, &hSCM  );
+	rpccli_svcctl_close_service(pipe_hnd, mem_ctx, &hSCM  );
 		
 	return werror_to_ntstatus(result);
 }	
@@ -180,9 +193,13 @@ done:
 /********************************************************************
 ********************************************************************/
 
-static NTSTATUS rpc_service_status_internal( const DOM_SID *domain_sid, const char *domain_name, 
-                                           struct cli_state *cli, TALLOC_CTX *mem_ctx, 
-                                           int argc, const char **argv )
+static NTSTATUS rpc_service_status_internal(const DOM_SID *domain_sid,
+						const char *domain_name, 
+						struct cli_state *cli,
+						struct rpc_pipe_client *pipe_hnd,
+						TALLOC_CTX *mem_ctx, 
+						int argc,
+						const char **argv )
 {
 	POLICY_HND hSCM, hService;
 	WERROR result = WERR_GENERAL_FAILURE;
@@ -200,27 +217,27 @@ static NTSTATUS rpc_service_status_internal( const DOM_SID *domain_sid, const ch
 
 	/* Open the Service Control Manager */
 	
-	result = cli_svcctl_open_scm( cli, mem_ctx, &hSCM, SC_RIGHT_MGR_ENUMERATE_SERVICE  );
+	result = rpccli_svcctl_open_scm(pipe_hnd, mem_ctx, &hSCM, SC_RIGHT_MGR_ENUMERATE_SERVICE  );
 	if ( !W_ERROR_IS_OK(result) ) {
-		d_printf("Failed to open Service Control Manager.  [%s]\n", dos_errstr(result));
+		d_fprintf(stderr, "Failed to open Service Control Manager.  [%s]\n", dos_errstr(result));
 		return werror_to_ntstatus(result);
 	}
 	
 	/* Open the Service */
 	
-	result = cli_svcctl_open_service( cli, mem_ctx, &hSCM, &hService, servicename, 
+	result = rpccli_svcctl_open_service(pipe_hnd, mem_ctx, &hSCM, &hService, servicename, 
 		(SC_RIGHT_SVC_QUERY_STATUS|SC_RIGHT_SVC_QUERY_CONFIG) );
 
 	if ( !W_ERROR_IS_OK(result) ) {
-		d_printf("Failed to open service.  [%s]\n", dos_errstr(result));
+		d_fprintf(stderr, "Failed to open service.  [%s]\n", dos_errstr(result));
 		goto done;
 	}
 	
 	/* get the status */
 
-	result = cli_svcctl_query_status( cli, mem_ctx, &hService, &service_status  );
+	result = rpccli_svcctl_query_status(pipe_hnd, mem_ctx, &hService, &service_status  );
 	if ( !W_ERROR_IS_OK(result) ) {
-		d_printf("Query status request failed.  [%s]\n", dos_errstr(result));
+		d_fprintf(stderr, "Query status request failed.  [%s]\n", dos_errstr(result));
 		goto done;
 	}
 	
@@ -228,15 +245,16 @@ static NTSTATUS rpc_service_status_internal( const DOM_SID *domain_sid, const ch
 
 	/* get the config */
 
-	result = cli_svcctl_query_config( cli, mem_ctx, &hService, &config  );
+	result = rpccli_svcctl_query_config(pipe_hnd, mem_ctx, &hService, &config  );
 	if ( !W_ERROR_IS_OK(result) ) {
-		d_printf("Query config request failed.  [%s]\n", dos_errstr(result));
+		d_fprintf(stderr, "Query config request failed.  [%s]\n", dos_errstr(result));
 		goto done;
 	}
 
 	/* print out the configuration information for the service */
 
 	d_printf("Configuration details:\n");
+	d_printf("\tControls Accepted    = 0x%x\n", service_status.controls_accepted);
 	d_printf("\tService Type         = 0x%x\n", config.service_type);
 	d_printf("\tStart Type           = 0x%x\n", config.start_type);
 	d_printf("\tError Control        = 0x%x\n", config.error_control);
@@ -268,19 +286,22 @@ static NTSTATUS rpc_service_status_internal( const DOM_SID *domain_sid, const ch
 	}
 
 done:	
-	cli_svcctl_close_service( cli, mem_ctx, &hService  );
-	cli_svcctl_close_service( cli, mem_ctx, &hSCM  );
-		
+	rpccli_svcctl_close_service(pipe_hnd, mem_ctx, &hService  );
+	rpccli_svcctl_close_service(pipe_hnd, mem_ctx, &hSCM  );
+
 	return werror_to_ntstatus(result);
 }	
 
-
 /********************************************************************
 ********************************************************************/
 
-static NTSTATUS rpc_service_stop_internal( const DOM_SID *domain_sid, const char *domain_name, 
-                                           struct cli_state *cli, TALLOC_CTX *mem_ctx, 
-                                           int argc, const char **argv )
+static NTSTATUS rpc_service_stop_internal(const DOM_SID *domain_sid,
+					const char *domain_name, 
+					struct cli_state *cli,
+					struct rpc_pipe_client *pipe_hnd,
+					TALLOC_CTX *mem_ctx, 
+					int argc,
+					const char **argv )
 {
 	POLICY_HND hSCM;
 	WERROR result = WERR_GENERAL_FAILURE;
@@ -295,16 +316,16 @@ static NTSTATUS rpc_service_stop_internal( const DOM_SID *domain_sid, const char
 
 	/* Open the Service Control Manager */
 	
-	result = cli_svcctl_open_scm( cli, mem_ctx, &hSCM, SC_RIGHT_MGR_ENUMERATE_SERVICE  );
+	result = rpccli_svcctl_open_scm(pipe_hnd, mem_ctx, &hSCM, SC_RIGHT_MGR_ENUMERATE_SERVICE  );
 	if ( !W_ERROR_IS_OK(result) ) {
-		d_printf("Failed to open Service Control Manager.  [%s]\n", dos_errstr(result));
+		d_fprintf(stderr, "Failed to open Service Control Manager.  [%s]\n", dos_errstr(result));
 		return werror_to_ntstatus(result);
 	}
 	
-	result = control_service( cli, mem_ctx, &hSCM, servicename, 
+	result = control_service(pipe_hnd, mem_ctx, &hSCM, servicename, 
 		SVCCTL_CONTROL_STOP, SVCCTL_STOPPED );
 		
-	cli_svcctl_close_service( cli, mem_ctx, &hSCM  );
+	rpccli_svcctl_close_service(pipe_hnd, mem_ctx, &hSCM  );
 		
 	return werror_to_ntstatus(result);
 }	
@@ -312,9 +333,13 @@ static NTSTATUS rpc_service_stop_internal( const DOM_SID *domain_sid, const char
 /********************************************************************
 ********************************************************************/
 
-static NTSTATUS rpc_service_pause_internal( const DOM_SID *domain_sid, const char *domain_name, 
-                                           struct cli_state *cli, TALLOC_CTX *mem_ctx, 
-                                           int argc, const char **argv )
+static NTSTATUS rpc_service_pause_internal(const DOM_SID *domain_sid,
+					const char *domain_name, 
+					struct cli_state *cli,
+					struct rpc_pipe_client *pipe_hnd,
+					TALLOC_CTX *mem_ctx, 
+					int argc,
+					const char **argv )
 {
 	POLICY_HND hSCM;
 	WERROR result = WERR_GENERAL_FAILURE;
@@ -329,16 +354,16 @@ static NTSTATUS rpc_service_pause_internal( const DOM_SID *domain_sid, const cha
 
 	/* Open the Service Control Manager */
 	
-	result = cli_svcctl_open_scm( cli, mem_ctx, &hSCM, SC_RIGHT_MGR_ENUMERATE_SERVICE  );
+	result = rpccli_svcctl_open_scm(pipe_hnd, mem_ctx, &hSCM, SC_RIGHT_MGR_ENUMERATE_SERVICE  );
 	if ( !W_ERROR_IS_OK(result) ) {
-		d_printf("Failed to open Service Control Manager.  [%s]\n", dos_errstr(result));
+		d_fprintf(stderr, "Failed to open Service Control Manager.  [%s]\n", dos_errstr(result));
 		return werror_to_ntstatus(result);
 	}
 	
-	result = control_service( cli, mem_ctx, &hSCM, servicename, 
+	result = control_service(pipe_hnd, mem_ctx, &hSCM, servicename, 
 		SVCCTL_CONTROL_PAUSE, SVCCTL_PAUSED );
 		
-	cli_svcctl_close_service( cli, mem_ctx, &hSCM  );
+	rpccli_svcctl_close_service(pipe_hnd, mem_ctx, &hSCM  );
 		
 	return werror_to_ntstatus(result);
 }	
@@ -346,9 +371,13 @@ static NTSTATUS rpc_service_pause_internal( const DOM_SID *domain_sid, const cha
 /********************************************************************
 ********************************************************************/
 
-static NTSTATUS rpc_service_resume_internal( const DOM_SID *domain_sid, const char *domain_name, 
-                                           struct cli_state *cli, TALLOC_CTX *mem_ctx, 
-                                           int argc, const char **argv )
+static NTSTATUS rpc_service_resume_internal(const DOM_SID *domain_sid,
+					const char *domain_name, 
+					struct cli_state *cli,
+					struct rpc_pipe_client *pipe_hnd,
+					TALLOC_CTX *mem_ctx, 
+					int argc,
+					const char **argv )
 {
 	POLICY_HND hSCM;
 	WERROR result = WERR_GENERAL_FAILURE;
@@ -363,16 +392,16 @@ static NTSTATUS rpc_service_resume_internal( const DOM_SID *domain_sid, const ch
 
 	/* Open the Service Control Manager */
 	
-	result = cli_svcctl_open_scm( cli, mem_ctx, &hSCM, SC_RIGHT_MGR_ENUMERATE_SERVICE  );
+	result = rpccli_svcctl_open_scm(pipe_hnd, mem_ctx, &hSCM, SC_RIGHT_MGR_ENUMERATE_SERVICE  );
 	if ( !W_ERROR_IS_OK(result) ) {
-		d_printf("Failed to open Service Control Manager.  [%s]\n", dos_errstr(result));
+		d_fprintf(stderr, "Failed to open Service Control Manager.  [%s]\n", dos_errstr(result));
 		return werror_to_ntstatus(result);
 	}
 	
-	result = control_service( cli, mem_ctx, &hSCM, servicename, 
+	result = control_service(pipe_hnd, mem_ctx, &hSCM, servicename, 
 		SVCCTL_CONTROL_CONTINUE, SVCCTL_RUNNING );
 		
-	cli_svcctl_close_service( cli, mem_ctx, &hSCM  );
+	rpccli_svcctl_close_service(pipe_hnd, mem_ctx, &hSCM  );
 		
 	return werror_to_ntstatus(result);
 }	
@@ -380,9 +409,13 @@ static NTSTATUS rpc_service_resume_internal( const DOM_SID *domain_sid, const ch
 /********************************************************************
 ********************************************************************/
 
-static NTSTATUS rpc_service_start_internal( const DOM_SID *domain_sid, const char *domain_name, 
-                                           struct cli_state *cli, TALLOC_CTX *mem_ctx, 
-                                           int argc, const char **argv )
+static NTSTATUS rpc_service_start_internal(const DOM_SID *domain_sid,
+					const char *domain_name, 
+					struct cli_state *cli,
+					struct rpc_pipe_client *pipe_hnd,
+					TALLOC_CTX *mem_ctx, 
+					int argc,
+					const char **argv )
 {
 	POLICY_HND hSCM, hService;
 	WERROR result = WERR_GENERAL_FAILURE;
@@ -398,41 +431,41 @@ static NTSTATUS rpc_service_start_internal( const DOM_SID *domain_sid, const cha
 
 	/* Open the Service Control Manager */
 	
-	result = cli_svcctl_open_scm( cli, mem_ctx, &hSCM, SC_RIGHT_MGR_ENUMERATE_SERVICE  );
+	result = rpccli_svcctl_open_scm( pipe_hnd, mem_ctx, &hSCM, SC_RIGHT_MGR_ENUMERATE_SERVICE  );
 	if ( !W_ERROR_IS_OK(result) ) {
-		d_printf("Failed to open Service Control Manager.  [%s]\n", dos_errstr(result));
+		d_fprintf(stderr, "Failed to open Service Control Manager.  [%s]\n", dos_errstr(result));
 		return werror_to_ntstatus(result);
 	}
 	
 	/* Open the Service */
 	
-	result = cli_svcctl_open_service( cli, mem_ctx, &hSCM, &hService, 
+	result = rpccli_svcctl_open_service(pipe_hnd, mem_ctx, &hSCM, &hService, 
 		servicename, SC_RIGHT_SVC_START );
 
 	if ( !W_ERROR_IS_OK(result) ) {
-		d_printf("Failed to open service.  [%s]\n", dos_errstr(result));
+		d_fprintf(stderr, "Failed to open service.  [%s]\n", dos_errstr(result));
 		goto done;
 	}
 	
 	/* get the status */
 
-	result = cli_svcctl_start_service( cli, mem_ctx, &hService, NULL, 0 );
+	result = rpccli_svcctl_start_service(pipe_hnd, mem_ctx, &hService, NULL, 0 );
 	if ( !W_ERROR_IS_OK(result) ) {
-		d_printf("Query status request failed.  [%s]\n", dos_errstr(result));
+		d_fprintf(stderr, "Query status request failed.  [%s]\n", dos_errstr(result));
 		goto done;
 	}
 	
-	result = watch_service_state( cli, mem_ctx, &hSCM, servicename, SVCCTL_RUNNING, &state  );
+	result = watch_service_state(pipe_hnd, mem_ctx, &hSCM, servicename, SVCCTL_RUNNING, &state  );
 	
 	if ( W_ERROR_IS_OK(result) && (state == SVCCTL_RUNNING) )
 		d_printf("Successfully started service: %s\n", servicename );
 	else
-		d_printf("Failed to start service: %s [%s]\n", servicename, dos_errstr(result) );
+		d_fprintf(stderr, "Failed to start service: %s [%s]\n", servicename, dos_errstr(result) );
 	
 done:	
-	cli_svcctl_close_service( cli, mem_ctx, &hService  );
-	cli_svcctl_close_service( cli, mem_ctx, &hSCM  );
-		
+	rpccli_svcctl_close_service(pipe_hnd, mem_ctx, &hService  );
+	rpccli_svcctl_close_service(pipe_hnd, mem_ctx, &hSCM  );
+
 	return werror_to_ntstatus(result);
 }
 
@@ -499,7 +532,7 @@ static int net_help_service( int argc, const char **argv )
 	d_printf("net rpc service start <service>    Start a service\n");
 	d_printf("net rpc service stop <service>     Stop a service\n");
 	d_printf("net rpc service pause <service>    Pause a service\n");
-	d_printf("net rpc service resume <service>   Resume a paused a service\n");
+	d_printf("net rpc service resume <service>   Resume a paused service\n");
 	d_printf("net rpc service status <service>   View the current status of a service\n");
 	
 	return -1;
@@ -525,5 +558,3 @@ int net_rpc_service(int argc, const char **argv)
 		
 	return net_help_service( argc, argv );
 }
-
-
